/* global it, describe, isNaN */
/* eslint-disable no-prototype-builtins */
const assert = require('assert');
const isEmpty = require('lodash.isempty');
const objectCompact = require('../index.js');

describe('objectCompact', () => {
  it('still being an empty object', () => {
    const received = objectCompact({});
    assert.equal(
      typeof received,
      'object',
      'a compacted empty object is an object',
    );
    assert.equal(
      isEmpty(received),
      true,
      'a compacted empty object still empty',
    );
  });
  it("shouldn't remove not Nil values", () => {
    const compacted = objectCompact({
      foo: 'bar',
      bar: 'baz',
    });
    assert.equal(compacted.foo, 'bar', "foo property hasn't changed");
    assert.equal(compacted.bar, 'baz', "bar property hasn't changed");
  });
  it('should remove Nil values', () => {
    const compacted = objectCompact({
      foo: null,
      bar: undefined,
      foobar: true,
    });

    assert.equal(
      compacted.hasOwnProperty('foo'),
      false,
      'foo property has been removed',
    );
    assert.equal(
      compacted.hasOwnProperty('bar'),
      false,
      'bar property has been removed',
    );
    assert.equal(
      compacted.hasOwnProperty('foobar'),
      true,
      "foobar property hasn't beend removed",
    );
  });
  it('should remove Nil nested values', () => {
    const compacted = objectCompact({
      bar: {
        foo: true,
        bar: null,
      },
    });

    assert.equal(
      compacted.bar.hasOwnProperty('bar'),
      false,
      'bar.bar property has beend removed',
    );
    assert.equal(
      compacted.bar.hasOwnProperty('foo'),
      true,
      "bar.foo property hasn't beend removed",
    );
  });
  it('should remove second level nested Nil values', () => {
    const compacted = objectCompact({
      bar: {
        baz: {
          foo: true,
          bar: null,
        },
      },
    });

    assert.equal(
      compacted.bar.baz.hasOwnProperty('bar'),
      false,
      'bar.bar property has beend removed',
    );
    assert.equal(
      compacted.bar.baz.hasOwnProperty('foo'),
      true,
      "bar.foo property hasn't beend removed",
    );
  });
  it("shouldn't treat Arrays as objects", () => {
    const compacted = objectCompact({
      bar: [1, 2, 3],
    });

    assert.equal(
      Array.isArray(compacted.bar),
      true,
      'bar still being an Array',
    );
  });
  it('should allow to change the default predicate to remove properties', () => {
    const compacted = objectCompact(
      {
        foo: NaN,
        bar: true,
      },
      // eslint-disable-next-line
      x => !isNaN(x),
    );

    assert.equal(
      compacted.hasOwnProperty('foo'),
      false,
      'NaNs are being removed from the object with `(x) => !isNaN(x)` filter predicate',
    );
    assert.equal(
      compacted.hasOwnProperty('bar'),
      true,
      'Properties that match the predicate are not being filtered out',
    );
  });
});
